<?php
namespace Geral\Util;


/**
 * Classe de captacao e exibicao de mensagem.
 *
 * Alguns processos desenvolvidos podem ter varios "passos", e estes podem ter de retornar varias mensagens. Por
 * isto, e necessario que haja uma forma de transitar estas mensagens.
 * Esta classe sera responsavel por acumular todas as mensagens produzidas pelo processo e, no final, exibi-las.
 * Ela sera implementada no padrao singleton.
 *
 * Created by JetBrains PhpStorm.
 * User: paulopereira
 * Date: 12/11/12
 * Time: 08:46
 * To change this template use File | Settings | File Templates.
 *
 */
class AcumuladorDeMensagens
{
    /**
     * Identificador de Acumulador de Mensagens. Usado para informar a classe de tratamento de excecoes que a excessao
     * enviada se trata de um objeto de Acumulador de Mensagens, e este deve trata-lo como tal.
     */
    const ID_ACUMULADOR_DE_MENSAGENS = "ae371e76160f02fa7563df6e0819fe5bff5c6b8d";

    /**
     * @var array Responsavel por acumular todas as mensagens produzidas.
     */
    private $acumulador;
    /**
     * @var int Identifica a quantidade de mensagens inseridas
     */
    private $contador;
    /** @var $contadorErros int contabiliza quantas mensagens sao mensagens de erros. */
    private $contadorErros;

    /** @var $instancia AcumuladorDeMensagens */
    private static $instancia;

    /**
     *
     */
    private function __construct()
    {
        $this->acumulador = array();
        $this->contador   = 0;
    }

    /**
     * Retorna uma instancia unica deste objeto. Isto garante que o acumulador de mensagens se torne global,
     * permitindo que várias classes enviem mensagens através deste.
     *
     * @static
     * @return AcumuladorDeMensagens
     */
    public static function getInstance()
    {
        if (self::$instancia === null) {
            self::$instancia = new self();
        }

        return self::$instancia;
    }

    /**
     * @param $texto
     * @param null $parametros
     * @param string $tipoMensagem
     */
    public function addMensagem($texto, $parametros = array(), $tipoMensagem = \Geral\Util\Mensagem::ALERTA)
    {
        $mensagem = new \Geral\Util\Mensagem();
        $mensagem->setTexto($texto);
        $mensagem->setParametros($parametros);
        $mensagem->setTipoMensagem($tipoMensagem);

        $this->acumulador[$this->contador] = $mensagem;
        $this->contador++;
        if ($tipoMensagem === Mensagem::ERRO) {
            $this->contadorErros++;
        }
    }

    /**
     * Remove o primeiro elemento da lista de mensagens do acumulador de mensagens.
     *
     * @return Mensagem
     */
    public function getMensagem()
    {
        /** @var $mensagem Mensagem */
        $mensagem = array_shift($this->acumulador);
        $this->contador--;
        if ($mensagem->getTipoMensagem() === Mensagem::ERRO) {
            $this->contadorErros--;
        }

        return $mensagem;
    }

    /**
     * Verifica se existem mensagens de erros entre as mensagens acumuladas.
     *
     * @return bool
     */
    public function existeMensagemErro()
    {
        return $this->contadorErros > 0;
    }

    /**
     * Verifica se ainda existem mensagens a serem impressas.
     *
     * @return bool
     */
    public function existemMensagens()
    {
        return $this->contador > 0;
    }

    /**
     * Compara o parâmetro texto com o texto de todas as mensagens.
     * Se encontrar, a mensagem já está inserida, então retorna TRUE.
     * Em caso contrário, retorna FALSE.
     *
     * @param String $texto Texto
     *
     * @return bool.
     */
    public function mensagemExiste ($texto)
    {
        foreach ($this->acumulador as $mensagem) {
            if ($mensagem instanceof \Geral\Util\Mensagem) {
                if ($mensagem->getTexto() == $texto) {
                    return true;
                }
            }
        }

        return false;
    }

}